#ifndef __CSTRING_H__
#define __CSTRING_H__

//---------------------------------------------------------------------------
//
//	File:			CSTRING.H
//	Programmer:		Bob Provencher
//	Create Date:	6/7/96
//	Last Mod Date:	6/7/96
//	Description:	string class emulation
//
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

class string
{

// attributes

private:

	char* chars;
	int len;
	int	size;

// construction/destruction

public:

	string( const char* src = 0 ) :
		chars( 0 ),
		len( 0 ),
		size( 0 )
	{
		Copy( src );
	}

	string( const string& src ) :
		chars( 0 ),
		len( 0 ),
		size( 0 )
	{
		Copy( src );
	}

	void Copy( const char* src, int new_len = -1 )
	{
		if ( src )
		{
			if ( new_len < 0 )
				new_len = strlen( src );
			if ( new_len + 1 > size )
				ResizeTo( new_len + 1 );
			memcpy( chars, src, new_len );
			chars[ new_len ] = 0;
			len = new_len;
		}
	}

	void Copy( const string& src )
	{
		Copy( src.c_str(), src.len );
	}

	void ResizeTo( int new_size )
	{
		char* new_chars = 0;
		if ( new_size > 0 )
			new_chars = new char[ new_size ];
		if ( new_chars )
		{
			memcpy( new_chars, chars, min( size, new_size ) );
			if ( new_size < size )
				new_chars[ new_size - 1 ] = 0;
		}
		delete chars;
		chars = new_chars;
		size = new_size;
		if ( size <= len )
			len = size - 1;
	}

	const char* c_str() const
	{
		return chars;
	}

	string& operator = ( const string& s )
	{
		Copy( s );
		return *this;
	}

	void to_upper()
	{
		char* p = chars;
		while( p && *p )
		{
			*p = (char)toupper( *p );
			p++;
		}
	}

	void operator += ( const string& s )
	{
		int new_size = len + s.len + 1;
		if ( new_size > size )
			ResizeTo( new_size );
		strcat( chars, s.chars );
		len += s.len;
	}

	string operator + ( const string& s2 ) const
	{
		string s1( *this );
		s1 += s2;
		return s1;
	}

};

#endif