#ifndef __DDCOM_H__
#define __DDCOM_H__

//---------------------------------------------------------------------------
//
//	File:			DDCOM.H
//	Programmer:		Bob Provencher
//	Create Date:	4/27/96
//	Last Mod Date:	4/27/96
//	Description:	DirectDraw COM object wrapper
//
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

//
//	provides release on destruction of COM objects
//

template <class T>
class DDComObject
{

private:

	T obj;	

	//
	//	manipulate reference counts
	//

protected:

	void AddRef()
	{
		if ( obj )
			obj->AddRef();
	}

	void Release()
	{
		if ( obj )
			obj->Release();
	}

	void Copy( const DDComObject< T >& dd )
	{
		Copy( dd.obj );
	}

	void Copy( T p )
	{
		Release();
		obj = p;
		AddRef();
		ASSERT_VALID( this );
	}

    T& Object()
    {
    	return obj;
    }

public:

	//
	//	initial creation, object just created - no AddRef()
	//

	DDComObject( T p = 0 ) :
		obj( p )
	{
	}

	//
	//	copy from another object
	//

	DDComObject( const DDComObject< T >& dd ) :
		obj( 0 )
	{
		Copy( dd );
		ASSERT_VALID( this );
	}

	//
	//	destructor, decrement use count
	//

	virtual ~DDComObject();

	//
	//	use this object for API function
	//	DON'T call Release or AddRef with this
	//

	T operator -> ()
	{
		ASSERT_VALID( this );
		return obj;
	}

	operator T () const
	{
		return obj;
	}

    T Object() const
    {
        return obj;
    }

	//
	//	copy other object into this one
	//

	DDComObject& operator = ( T p )
	{
		ASSERT_VALID( this );
		Copy( p );
		return *this;
	}

	//
	//	copy other object into this one
	//

	DDComObject& operator = ( const DDComObject< T >& dd )
	{
		ASSERT_VALID( this );
		Copy( dd );
		return *this;
	}

	#ifdef _DEBUG
	virtual void AssertValid() const;
	#endif

};

//
//	destructor, decrement use count
//

template <class T>
DDComObject< T >::~DDComObject()
{
	Release();
}

#ifdef _DEBUG
template <class T>
void DDComObject< T >::AssertValid() const
{
	ASSERT( obj != 0 );
}
#endif

#endif

