#ifndef __DDDIAG_H__
#define __DDDIAG_H__

//---------------------------------------------------------------------------
//
//	File:			DDDIAG.H
//	Programmer:		Bob Provencher
//	Create Date:	7/15/95
//	Last Mod Date:	4/26/96
//	Description:	Direct Draw diagnostic messages and helpers
//
//	Copyright (c) 1995 Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef __EXCEPT_H__
#include "Except.h"
#endif

extern HRESULT hResult;

const char* GetErrorDesc( HRESULT hRes );

void Message( char* msg, UINT style = MB_OK | MB_ICONINFORMATION );

struct ErrorDesc
{
	HRESULT	hRes;
	char*	sResStr;
};

class xDDFail : public Exception
{

private:

	HRESULT	hResult;
	string	sCodeStr;

public:

	xDDFail( HRESULT h,
			 const string& file = unknown,
			 int line = -1 ) :
		Exception( "DirectDraw interface failure", file, line ),
		hResult( h )
	{
		sCodeStr = GetErrorDesc( hResult );
	}

	HRESULT GetHResult() const
	{
		return hResult;
	}

	UINT GetCode() const
	{
		return SCODE_CODE( hResult );
	}

	string GetCodeStr() const
	{
		return sCodeStr;
	}

	string Desc() const;

};

#if defined( _DEBUG )

void AssertSuccess( HRESULT hRes, const string& file = unknown, int line = -1 );
void AssertValidDDRect( const RECT& rc, const string& file = unknown, int line = -1 );

#define ASSERT_SUCCESS( hRes )			{	AssertSuccess( hRes, __FILE__, __LINE__ );		}
#define ASSERT_VALID_DD_RECT( rc )		{	AssertValidDDRect( rc, __FILE__, __LINE__ );		}
#define THROWDD( api, hRes )			{	throw( new xDDFail( api, hRes, __FILE__, __LINE__ ) );	}
#define TRACEDD( a )					{ 	TRACE( __FILE__ " line " << __LINE__ << ": " << a ); }

#else

#define ASSERT_SUCCESS( api, hRes )		__VOID0
#define ASSERT_VALID_DD_RECT( rc )		__VOID0
#define THROWDD( api, hRes )			__VOID0
#define TRACEDD( a ) 					__VOID0

#endif

#endif

