#ifndef __DDOBJ_H__
#define __DDOBJ_H__

//---------------------------------------------------------------------------
//
//	File:			DDOBJ.H
//	Programmer:		Bob Provencher
//	Create Date:	4/27/96
//	Last Mod Date:	4/27/96
//	Description:	DirectDraw object wrapper
//
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef __COMMON_H__
#include "Common.h"
#endif

#ifndef __DDCOM_H__
#include "DDCom.h"
#endif

#ifndef __DDSURF_H__
#include "DDSurf.h"
#endif

#ifndef __DDCLIP_H__
#include "DDClip.h"
#endif

#ifndef __DDPAL_H__
#include "DDPal.h"
#endif

#ifndef __DDDIAG_H__
#include "DDDiag.h"
#endif

class DDObject : public DDComObject< LPDIRECTDRAW >
{

public:

	//
	//	initial creation, object just created
	//

	DDObject( LPDIRECTDRAW p = 0 ) :
		DDComObject< LPDIRECTDRAW >( p )
	{
	}

	//
	//	copy from another object
	//

	DDObject( const DDObject& dd ) :
		DDComObject< LPDIRECTDRAW >( dd )
	{
		ASSERT_VALID( this );
	}

	DDObject& operator = ( const DDObject& dd )
	{
		Copy( dd );
        ASSERT_VALID( this );
		return *this;
	}

    void Create()
    {
    	LPDIRECTDRAW lp;
        DirectDrawCreate( NULL, &lp, NULL );
        Copy( lp );
    	ASSERT_VALID( this );
    }

    void CreateSurface( DDSURFACEDESC& desc, DDSurface& surf )
    {
    	ASSERT_VALID( this );
        LPDIRECTDRAWSURFACE lp;
        HRESULT hRes = Object()->CreateSurface( &desc, &lp, NULL );
        ASSERT_SUCCESS( hRes );
        surf.Release();
    	surf.Object() = lp;
        ASSERT_VALID( &surf );
    }

    void CreateClipper( DDClipper& clip )
    {
    	ASSERT_VALID( this );
        LPDIRECTDRAWCLIPPER lp;
        HRESULT hRes = Object()->CreateClipper( 0, &lp, NULL );
        ASSERT_SUCCESS( hRes );
        clip.Release();
    	clip.Object() = lp;
        ASSERT_VALID( &clip );
    }

    void CreatePalette( DWORD dwFlags, LPPALETTEENTRY lpPal, DDPalette& pal )
    {
    	ASSERT_VALID( this );
        LPDIRECTDRAWPALETTE lp;
        HRESULT hRes = Object()->CreatePalette( dwFlags, lpPal, &lp, 0 );
        ASSERT_SUCCESS( hRes );
        pal.Release();
    	pal.Object() = lp;
        ASSERT_VALID( &pal )
    }

	#ifdef _DEBUG
	void AssertValid() const;
	#endif

};

#endif

