#ifndef __DEBUG_H__
#define __DEBUG_H__

//---------------------------------------------------------------------------
//
//	File:			DDDIAG.H
//	Programmer:		Bob Provencher
//	Create Date:	4/26/96
//	Last Mod Date:	4/26/96
//	Description:	compiler independent diagnostic message macros
//
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef __EXCEPT_H__
#include "Except.h"
#endif

#if defined( __BORLANDC__ ) && defined( __DEBUG )
#define _DEBUG
#endif

#if defined( _DEBUG )

#include <fstream.h>

extern ofstream debug;
extern BOOL trace;
extern BOOL _break;

void Assert( BOOL b, const string stmt, const string file = unknown,
			 int line = -1 );

#if defined( _MSC_VER )
#define BREAK						{   if ( _break ) { __asm int 3			} }
#else
#define BREAK						{   if ( _break ) { __emit__( 0xCC );	} }
#endif

#define SETBRK( b )					{	_break = b;							}
#define BREAKON						SETBRK( TRUE  )
#define BREAKOFF					SETBRK( FALSE )

#define TRACE( a )					{ 	if ( trace ) { debug << a; debug.flush(); } }
#define SETTRACE( b )				{ 	trace = b;								}
#define TRACEON						SETTRACE( TRUE  )
#define TRACEOFF					SETTRACE( FALSE )

#define ASSERT( a )					{	Assert( (a), #a, __FILE__, __LINE__ );	}
#define ASSERT_VALID( p )   		{	(p)->AssertValid();						}
#define ASSERT_VALID_OR_NULL( p )   {	if ( p ) p->AssertValid();				}

#else

#define __VOID0						((void)0)

#define BREAK						__VOID0
#define SETBREAK( b )				__VOID0
#define BREAKON						__VOID0
#define BREAKOFF					__VOID0
#define TRACE( a )					__VOID0
#define SETTRACE( b )				__VOID0
#define TRACEON						__VOID0
#define TRACEOFF					__VOID0
#define ASSERT( a )					__VOID0
#define ASSERT_VALID( a )			__VOID0
#define ASSERT_VALID_OR_NULL( a )	__VOID0

#endif

#endif
