//---------------------------------------------------------------------------
//
//	File:			DIB.CPP
//	Programmer: 	Bob Provencher
//	Create Date:	11/9/94
//	Last Mod Date:	4/26/96
//	Description:	Dib class implementation, formerly BMPINFO.CPP
//
//	Copyright (c) 1994, 1995. Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "SysInc.h"
#pragma hdrstop

#ifndef __DIB_H__
#include "Dib.h"
#endif

#ifndef __RES_H__
#include "Res.h"
#endif

COLORREF RgbquadToColorref( const RGBQUAD& rgbq )
{
	return RGB( rgbq.rgbRed, rgbq.rgbGreen, rgbq.rgbBlue );
}

RGBQUAD ColorrefToRgbquad( const COLORREF& colorref )
{
	RGBQUAD rgbq;
	rgbq.rgbRed			= GetRValue( colorref );
	rgbq.rgbGreen		= GetGValue( colorref );
	rgbq.rgbBlue		= GetBValue( colorref );
	rgbq.rgbReserved	= 0;
	return rgbq;
}

void Dib::Cleanup()
{
	if ( lpBmi )
	{
		ASSERT_VALID( this );
		GlobalFreePtr( lpBmi );
		lpBmi = 0;
	}
}

//
//	offset to the next scan line
//

DWORD Dib::GetPitch( LPBITMAPINFOHEADER lp ) const
{
	if ( !lp )
		lp = GetBitmapInfoHeader();
	ASSERT( lp != 0 );
	// BOBTODO: not sure about this
	return ( ( ( GetWidth( lp ) * GetBitsPerPixel( lp ) ) / BITS_PER_BYTE ) + 3 ) & ~3;
}

//
//	pointer to the bitmap bits
//

LPBYTE Dib::GetBitmap( LPBITMAPINFOHEADER lp ) const
{
	if ( !lp )
		lp = GetBitmapInfoHeader();
	ASSERT( lp != 0 );
	return (LPBYTE)lp + GetImageOffset( lp );
}

//
//	initialize the memory for a DIB and copy in the other DIB
//

void Dib::Init( LPBITMAPINFO lp, RGBQUAD* colors, LPBYTE bits )
{
	LPBITMAPINFO lpbi;
	if ( !lp )
		return;
	lpbi = (LPBITMAPINFO)GlobalAllocPtr( GHND, GetPackedDibSize( (LPBITMAPINFOHEADER)lp ) );
	if ( GlobalSize( GlobalHandle( lpbi ) ) != GlobalSize( GlobalHandle( lp ) ) )
		BREAK;
	if ( lpbi )
	{
		memcpy( lpbi, lp, sizeof( BITMAPINFOHEADER ) );
		// BOBTODO: seh here to detect no color table/image?
		DWORD size = GetColorTableSizeBytes( (LPBITMAPINFOHEADER)lp );
		if ( size )
		{
			if ( !colors )
				colors = GetColorTable( lp );
			memcpy( GetColorTable( lpBmi ), colors, size );
		}
		size = GetImageBytes( (LPBITMAPINFOHEADER)lp );
		if ( size )
		{
			if ( !bits )
				bits = GetBitmap( (LPBITMAPINFOHEADER)lp );
			memcpy( GetBitmap( (LPBITMAPINFOHEADER)lpbi ), bits, size );
		}
	}
	if ( lpBmi )
		Cleanup();
	lpBmi = lpbi;
	// HACK
	if ( GetBitCount() == 8 )
	{
		RGBQUAD* colors = GetColorTable();
		colors[ 0 ] = ColorrefToRgbquad( RGB( 0, 0, 0 ) );
		int num = GetNumColors();
		if ( num == 255 )
			colors[ 255 ] = ColorrefToRgbquad( RGB( 255, 255, 255 ) );
	}
}

//
//	conversion routines
//

//
//	GetBits: Convert an HBITMAP to a DIB using GetDIBits
//

void Dib::GetBits( HBITMAP hBmp, HPALETTE& hPal )
{

	BITMAP bm;					// bitmap structure
	BITMAPINFOHEADER bih;		// bitmapinfoheader
	LPBITMAPINFOHEADER lpbih;	// pointer to BITMAPINFOHEADER
	WORD biBits;				// bits per pixel

	ASSERT( hBmp != 0 );

	if ( GetObject( hBmp, sizeof( BITMAP ), &bm ) )
	{
    	THROW( xAPIFail, "GetObject" );
	}

	/* calculate bits per pixel */
	biBits = (WORD)( bm.bmPlanes * bm.bmBitsPixel );

	/* make sure bits per pixel is valid */
	if ( biBits <= 1 )
		biBits = 1;
	else if ( biBits <= 4 )
		biBits = 4;
	else if ( biBits <= 8 )
		biBits = 8;
	else /* if greater than 8-bit, force to 24-bit */
		biBits = 24;

	/* initialize BITMAPINFOHEADER */

	memset( &bih, 0, sizeof( BITMAPINFOHEADER ) );

	bih.biSize			= sizeof( BITMAPINFOHEADER );
	bih.biWidth			= bm.bmWidth;
	bih.biHeight		= bm.bmHeight;
	bih.biPlanes		= 1;
	bih.biBitCount		= biBits;
	bih.biCompression	= BI_RGB;

	lpbih = (LPBITMAPINFOHEADER)GlobalAllocPtr( GHND, GetPackedDibSize( &bih ) );

	if ( !lpbih )
		return;

	/* get a DC */

	HDC hDC = CreateCompatibleDC( 0 );

	BOOL fPalDevice = GetDeviceCaps( hDC, RASTERCAPS ) & RC_PALETTE;

	/* if no palette is specified, use default palette */
	if ( fPalDevice )
	{
		if ( !hPal )
			hPal = (HPALETTE)GetStockObject( DEFAULT_PALETTE );

		/* select and realize our palette */
		hPal = SelectPalette( hDC, hPal, FALSE );
		RealizePalette( hDC );
	}

	/*  
	 *  get the bits
	 */
	if ( ::GetDIBits( hDC, hBmp, 0, (WORD)bih.biHeight,
		 GetBitmap( lpbih ), (LPBITMAPINFO)lpbih, DIB_RGB_COLORS ) == 0 )
	{
		GlobalFreePtr( lpbih );
		THROW( xAPIFail, "GetDIBits" );
	}
	else
	{
		if ( lpBmi )
			Cleanup();
		lpBmi = (LPBITMAPINFO)lpbih;
	}

	if ( fPalDevice )
	{
		SelectPalette( hDC, hPal, TRUE );
		RealizePalette( hDC );
	}

	ASSERT_VALID( this );

}

//
//	convert a DIB to an HBITMAP using SetDIBits
//

void Dib::SetBits( HBITMAP& hBmp ) const
{
	ASSERT_VALID( this );
	if ( hBmp )
		CreateBitmap( hBmp );
	HDC hDC = ::GetDC( 0 );
	::SetDIBits( hDC, hBmp, 0, GetHeight(),
				 GetBitmap(), GetBitmapInfo(), DIB_RGB_COLORS );
	::ReleaseDC( 0, hDC );
}

void Dib::SetMaskBits( HBITMAP& hBmp, COLORREF trans ) const
{

	ASSERT_VALID( this );

	if ( !hBmp )
		CreateMaskBitmap( hBmp, trans );

	ASSERT( GetBitCount() == 8 );

	DWORD size;
	LPBYTE lpByte;

	if ( ( size = GetImageBytes() ) > 0 )
	{
		if ( ( lpByte = (LPBYTE)GlobalAllocPtr( GHND, size ) ) != 0 )
		{

			memcpy( lpByte, GetBitmap(), size );

			BYTE trans_index;
            FindColor( trans, trans_index );

			for ( int i = 0; i < (int)size; i++ )
				lpByte[ i ] = lpByte[ i ] == trans_index ? (BYTE)255 : (BYTE)0;

			HDC hDC = ::GetDC( 0 );
			::SetDIBits( hDC, hBmp, 0, GetHeight(),
						 lpByte, GetBitmapInfo(), DIB_RGB_COLORS );
			::ReleaseDC( 0, hDC );

			GlobalFreePtr( lpByte );

		}
	}
}

//
//	put a DIBs bits into a newly created bitmap's (DDB) bits
//

void Dib::CreateBitmap( HBITMAP& hBmp ) const
{

	ASSERT_VALID( this );

	if ( hBmp )
		SetBits( hBmp );

	HDC hDC = ::GetDC( 0 );

	HBITMAP hBmp1 = ::CreateDIBitmap( hDC, GetBitmapInfoHeader(), CBM_INIT, GetBitmap(),
					  GetBitmapInfo(), DIB_RGB_COLORS );

	::ReleaseDC( 0, hDC );

	if ( hBmp1 )
		hBmp = hBmp1;
    else
    {
    	THROW( xAPIFail, "CreateDIBitmap" );
    }

}

void Dib::CreateMaskBitmap( HBITMAP& hBmp, COLORREF trans ) const
{

	ASSERT_VALID( this );

	if ( hBmp )
		SetBits( hBmp );

	DWORD size;
	LPBYTE lpByte;

	if ( ( size = GetImageBytes() ) > 0 )
	{
		if ( ( lpByte = (LPBYTE)GlobalAllocPtr( GHND, size ) ) != 0 )
		{

			memcpy( lpByte, GetBitmap(), size );

			BYTE trans_index;
            FindColor( trans, trans_index );

			for ( int i = 0; i < (int)size; i++ )
				lpByte[ i ] = lpByte[ i ] == trans_index ? (BYTE)255 : (BYTE)0;

			HDC hDC = ::GetDC( 0 );
			HBITMAP hBmp1 = ::CreateDIBitmap( hDC, GetBitmapInfoHeader(), CBM_INIT, lpByte,
							  GetBitmapInfo(), DIB_RGB_COLORS );
			::ReleaseDC( 0, hDC );

			GlobalFreePtr( lpByte );

			if ( hBmp1 )
				hBmp = hBmp1;
            else
            {
            	THROW( xAPIFail, "CreateDIBitmap" );
            }

		}

	}
}

//
//	Find a color in the color table
//

void Dib::FindColor( COLORREF color, BYTE& byte ) const
{

	ASSERT( GetBitCount() == 8 );

	RGBQUAD* colors = GetColorTable();
	int num = (int)GetNumColors();

	for ( int i = 0; i < num; i++ )
	{
		if ( RgbquadToColorref( colors[ i ] ) == color )
        {
			byte = (BYTE)i;
            return;
        }
	}

	THROW( xNotFound, "Dib::FindColor" );

}

//
//	Update every instance of one color with another color
//

void Dib::UpdateColors( COLORREF color, COLORREF new_color )
{

	ASSERT_VALID( this );
	DWORD size;
	LPBYTE lpByte;
	BYTE color_index;
	BYTE new_color_index;
    FindColor( color, color_index );
	FindColor( new_color, new_color_index );

	if ( ( size = GetImageBytes() ) > 0 )
	{
		if ( ( lpByte = GetBitmap() ) != 0 )
		{
			for ( DWORD i = 0; i < size; i++ )
			{
				if ( lpByte[ i ] == color_index )
					lpByte[ i ] = new_color_index;
			}
		}
	}
}

//
//	put a DIBs bits into a newly created bitmap's (DDB) bits,
//	will be dynamically updated with DIB bits
//

void Dib::CreateSection( HBITMAP& hBmp ) const
{
	ASSERT( 0 );	// need to seperate BITMAPINFOHEADER from bits for this to work
	ASSERT_VALID( this );
	ASSERT( hBmp == 0 );
	HDC hDC = ::GetDC( 0 );
	HBITMAP hBmp1 = 0; // = ::CreateDIBSection( ... );
	::ReleaseDC( 0, hDC );
	if ( hBmp1 )
		hBmp = hBmp1;
    else
    {
    	THROW( xAPIFail, "CreateDIBSection" );
    }
}

//
//	put a DIB directly on a screen device
//

void Dib::SetBitsToDevice( HDC hDC,  const POINT& ptDest, const RECT& rcSrc ) const
{
	ASSERT_VALID( this );
	int num = ::SetDIBitsToDevice( hDC, ptDest.x, ptDest.y,
						 Width( rcSrc ), Height( rcSrc ), rcSrc.left, rcSrc.top,
						 0, GetHeight(), GetBitmap(), GetBitmapInfo(),
						 DIB_RGB_COLORS );
	if ( num == 0 )
	{
//		DWORD dwErr = ::GetLastError();
		THROW( xAPIFail, "SetDIBitsToDevice" );
	}
}

//
//	Load utilities
//

//
//	Load from a resource
//

void Dib::Load( UINT id )
{

	Resource dib( MAKEINTRESOURCE( id ), RT_BITMAP );


	LPBITMAPINFO lpBmi = (LPBITMAPINFO)dib.Res();
	Init( lpBmi );

}

//
//	Load from file
//

void Dib::Load( const string& filename )
{
	ifstream in( filename.c_str(), ios::in | ios::binary );
    if ( in )
		Load( in );
    else
    	THROW( xFileAccess, "File open error" ); 
}

void Dib::Load( ifstream& in )
{

	DWORD len;
	BITMAPFILEHEADER bmfh;
	LPBITMAPINFO lp;
	if ( ( len = Length( in ) ) != 0 )
	{
		if ( len > sizeof( BITMAPFILEHEADER ) + sizeof( BITMAPINFO ) )
		{
			len -= sizeof( BITMAPFILEHEADER );
			if ( ( lp = (LPBITMAPINFO)GlobalAllocPtr( GHND, len ) ) != 0 )
			{
				if ( in.tellg() != 0 )
					in.seekg( 0, ios::beg );
				in.read( (LPBYTE)&bmfh, sizeof( BITMAPFILEHEADER ) );
				if ( bmfh.bfType == MAKEWORD( 'B', 'M' ) )
				{
					in.read( (LPBYTE)lp, len );
					Cleanup();
					lpBmi = lp;
					return;
				}
				GlobalFreePtr( lp );
                THROW( xFile, "Invalid Bitmap File" );
			}
           	THROW( xAPIFail, "GlobalAlloc" );
		}
	}
	THROW( xFile, "Invalid Bitmap File" );
}

#ifdef _DEBUG
void Dib::AssertValid() const
{

	static BOOL fAsserting = FALSE;

	if ( fAsserting )
		return;

	fAsserting = TRUE;

	//
	//	gotta have a pointer
	//
	ASSERT( lpBmi != 0 );

	//
	//	we only support win 3.0 dibs
	//
	ASSERT( GetSize() == sizeof( BITMAPINFOHEADER ) );

	//
	//	only RGB for now
	//
    DWORD comp = GetCompression();
	ASSERT( comp == BI_RGB || comp == BI_RLE8 );

	//
	//	only support older known format bitmaps (we don't read BI_BITFIELDS yet)
	//
	WORD bpp = GetBitCount();

	ASSERT( bpp == 1 || bpp == 4 || bpp == 8 || bpp == 24 );

	fAsserting = FALSE;

}

void Dib::Dump( ostream& debug ) const
{
	debug << lpBmi;
}
#endif
