#ifndef __EXCEPT_H__
#define __EXCEPT_H__

//---------------------------------------------------------------------------
//
//	File:			EXCEPT.H
//	Programmer: 	Bob Provencher
//	Create Date:	1/23/94
//	Last Mod Date:	4/6/96
//	Description:	exception hierarchy
//
//	Copyright (c) 1994, 1995. Aesir Software, Inc.
//	Copyright (c) 1996. Cog Interactive.
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef __SYSINC_H__
#include "sysinc.h"
#endif

#ifndef __GEN_H__
#include "Gen.h"
#endif

#if defined( __BORLANDC__ ) && defined( __DEBUG )
#define _DEBUG
#endif

const string unknown( "unknown" );

class Exception
{

// construction/destruction

public:

	Exception( const string& i,
		   	   const string& f = unknown,
		   	   int l = -1 ) :
		info( i ),
		file( f ),
		line( l )
	{
    	file.to_upper();
		if ( line == -1 )
			line_str = unknown;
		else
	        itos( line, line_str );
	}

// attributes

protected:

	string info;
	string file;
	int line;
    string line_str;

public:

	const string& Info() const
	{
		return info;
	}

	const string& File() const
	{
		return file;
	}

	int Line() const
	{
		return line;
	}

	const string& LineStr() const
    {
        return line_str;
    }

	virtual string Desc() const;

	void MessageBox( UINT id = MB_OK | MB_ICONSTOP, HWND hWnd = 0 )
	{
		::MessageBox( hWnd, Desc().c_str(), "Exception", id );
	}

#ifdef _DEBUG
	void AssertValid() const;
	void Dump( ostream& out ) const;
#endif

};

class xRange : public Exception
{

public:

	xRange( const string& info,
			const string& file = unknown,
			int line = -1 ) :
		Exception( info, file, line )
	{
	}

};

class xCreate : public Exception
{

public:

	xCreate( const string& info,
			 const string& file = unknown,
			 int line = -1 ) :
		Exception( info, file, line )
	{
	}

};

class xNotFound : public Exception
{

public:

	xNotFound( const string& info,
			   const string& file = unknown,
			   int line = -1 ) :
		Exception( info, file, line )
	{
	}

};

class xFile : public Exception
{

public:

	xFile( const string& info,
			 const string& file = unknown,
			 int line = -1 ) :
		Exception( info, file, line )
	{
	}

};

class xFileAccess : public xFile
{

public:

	xFileAccess( const string& info,
			 	 const string& file = unknown,
			 	 int line = -1 ) :
		xFile( info, file, line )
	{
	}

};

class xAssertFail : public Exception
{

public:

	xAssertFail( const string& info,
			 	 const string& file = unknown,
			 	 int line = -1 ) :
		Exception( string( "Assert " ) + info + " failed", file, line )
	{
	}

};

class xAPIFail : public Exception
{

private:

	DWORD dwApiError;
	string sApiError;

public:

	xAPIFail( const string& info,
			  const string& file = unknown,
			  int line = -1 ) :
		Exception( info, file, line ),
		dwApiError( 0 )
	{
		dwApiError = ::GetLastError();
		itos( dwApiError, sApiError );
	}

	DWORD GetAPIError() const
	{
		return dwApiError;
	}

	const string& GetAPIErrorStr() const
	{
		return sApiError;		
	}

	string Desc() const;

};

#if defined( _DEBUG )
#define THROW( type, info ) 	{ 	throw( new type( (info), __FILE__, __LINE__ ) );	}
#else
#define THROW( type, info ) 	{	throw( new type( (info) ) );						}
#endif

#endif


