//---------------------------------------------------------------------------
//
//	File:			MAP.CPP
//	Programmer:		Bob Provencher
//	Create Date:	4/28/96
//	Last Mod Date:	4/28/96
//	Description:	Map structure
//
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "SysInc.h"
#pragma hdrstop

#ifndef __COMMON_H__
#include "Common.h"
#endif

#ifndef __MAP_H__
#include "Map.h"
#endif

#define STONE				10
#define MUD					20

Dir moves[ NUM_DIRS ] = 
{
	{  0, -1 },		// north
	{  1, -1 },		// ne
	{  1,  0 },		// east
	{  1,  1 },		// se
	{  0,  1 },		// south
	{ -1,  1 },		// sw
	{ -1,  0 },		// west
	{ -1, -1 }		// nw
};

void Map::Generate( int x, int y )
{

	ResizeTo( x, y );

    int roll;

    for ( int i = 0; i < x; i++ )
    	for ( int j = 0; j < y; j++ )
        {
			roll = random( 100 );
            if ( roll < STONE )
            	SetTile( i, j, stone );
			else if ( roll < MUD )
            	SetTile( i, j, mud );
            else
            	SetTile( i, j, grass );
        }

	SetMan( x / 2, y / 2 );

}

void Map::Copy( const Map& map )
{
	ResizeTo( map.Width(), map.Height() );
	for ( int i = 0; i < width; i++ )
		for ( int j = 0; j < height; j++ )
			GetTile( i, j ) = map[ i ][ j ];
	map.GetMan( man_x, man_y );
	ASSERT_VALID( this );
}

BOOL Map::IsValid( int x, int y ) const
{
	ASSERT_VALID( this );
	return x >= 0 && x < Width() && y >= 0 && y < Height();
}

BOOL Map::IsValidMove( int x, int y, DirType dir ) const
{
	ASSERT_VALID( this );
	ASSERT( IsValid( x, y ) );
	return IsValid( x + moves[ dir ].dx, y + moves[ dir ].dy );
}

#ifdef _DEBUG
static BOOL fInAssert = FALSE;
void Map::AssertValid() const
{
	if ( fInAssert )
		return;
	fInAssert = TRUE;
	ASSERT( tiles != 0 );
	ASSERT( width > 0 );
	ASSERT( height > 0 );
	ASSERT( IsValid( man_x, man_y ) );
	fInAssert = FALSE;
}
#endif
