#ifndef __MAP_H__
#define __MAP_H__

//---------------------------------------------------------------------------
//
//	File:			MAP.H
//	Programmer:		Bob Provencher
//	Create Date:	4/28/96
//	Last Mod Date:	4/28/96
//	Description:	Map structure
//
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef __COMMON_H__
#include "Common.h"
#endif

#ifndef __TILE_H__
#include "Tile.h"
#endif

enum DirType
{
	north = 0,
	north_east,
	east,
	south_east,
	south,
	south_west,
	west,
	north_west
};

const int NUM_DIRS = north_west - north + 1;

struct Dir
{
	int	dx;
	int dy;
};

extern Dir moves[ NUM_DIRS ];

class Map
{

private:

	int width;
	int height;

	TileType* tiles;

	int man_x;
	int man_y;

public:

	Map( int w, int h ) :
		width( 0 ),
		height( 0 ),
		tiles( 0 )
	{
		ResizeTo( w, h );
		ASSERT_VALID( this );
	}

	Map( const Map& map ) :
		width( 0 ),
		height( 0 ),
		tiles( 0 )
	{
		Copy( map );
	}

// attributes

	int Width() const
	{
		ASSERT_VALID( this );
		return width;
	}

	int Height() const
	{
		ASSERT_VALID( this );
		return height;
	}

	TileType* Tiles() const
	{
		ASSERT_VALID( this );
		return tiles;
	}

	BOOL IsValid( int x, int y ) const;
	BOOL IsValidMove( int x, int y, DirType dir ) const;

// methods

	void Generate( int x, int y );

	TileType& GetTile( int x, int y )
	{
		ASSERT_VALID( this );
		ASSERT( IsValid( x, y ) );
		return tiles[ ( x * height ) + y ];
	}

	const TileType& GetTile( int x, int y ) const
	{
		ASSERT_VALID( this );
		ASSERT( IsValid( x, y ) );
		return tiles[ ( x * height ) + y ];
	}

	void SetTile( int x, int y, const TileType& tile )
	{
		ASSERT_VALID( this );
		ASSERT( IsValid( x, y ) );
		tiles[ ( x * height ) + y ] = tile;
	}

	void GetMan( int& x, int& y ) const
	{
		ASSERT_VALID( this );
		x = man_x;
		y = man_y;
	}

	void SetMan( int x, int y )
	{
		ASSERT_VALID( this );
		ASSERT( IsValid( x, y ) );
		man_x = x;
		man_y = y;
	}

	TileType* operator [] ( int x )
	{
		ASSERT_VALID( this );
		ASSERT( IsValid( x, 0 ) );
		return tiles + ( x * height );
	}

	const TileType* operator [] ( int x ) const
	{
		ASSERT_VALID( this );
		ASSERT( IsValid( x, 0 ) );
		return tiles + ( x * height );
	}

	void Copy( const Map& map );

	void ResizeTo( int w, int h )
	{
		TileType* t = new TileType[ w * h ];
		Cleanup();
		tiles  = t;
		width  = w;
		height = h;
		ASSERT_VALID( this );
	}

	void Cleanup()
	{
		delete [] tiles;
		width = height = 0;
		man_x = man_y = 0;
	}

#ifdef _DEBUG
	void AssertValid() const;
#endif

};

#endif

