#ifndef __RES_H__
#define __RES_H__

//--------------------------------------------------------------------------
//
//	File:			RES.H
//	Programmer:		Bob Provencher
//	Create Date: 	12/1/94
//	Last Mod Date:	4/26/96
//	Description:	Resource load class
//
//	Copyright (c) 1994, 1995. Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//--------------------------------------------------------------------------

#ifndef __SYSINC_H__
#include "SysInc.h"
#endif

#ifndef __GEN_H__
#include "Gen.h"
#endif

#ifndef __EXCEPT_H__
#include "Except.h"
#endif

class Resource
{

private:

	HRSRC		hRsrc;
	HGLOBAL		hGlobal;
	LPVOID		lpRes;

	static HINSTANCE   hInst;

	static HINSTANCE Inst()
	{
		return hInst;
	}

public:

	BOOL IsFound() const
	{
		return (BOOL)hRsrc;
	}

	BOOL IsLoaded() const
	{
		return (BOOL)hGlobal;
	}

	BOOL IsLocked() const
	{
		return (BOOL)lpRes;
	}

	LPVOID Res() const
	{
		return lpRes;
	}

	Resource() :
		hRsrc( 0 ),
		hGlobal( 0 ),
		lpRes( 0 )
	{
	}

	Resource( UINT uName, UINT uType ) :
		hRsrc( 0 ),
		hGlobal( 0 ),
		lpRes( 0 )
	{
		Load( MAKEINTRESOURCE( uName ), MAKEINTRESOURCE( uType ) );
		Lock();
	}

	Resource( LPCSTR lpszName, LPCSTR lpszType ) :
		hRsrc( 0 ),
		hGlobal( 0 ),
		lpRes( 0 )
	{
		Load( lpszName, lpszType );
		Lock();
	}

	~Resource()
	{
		if ( IsLocked() )
			Unlock();
		if ( IsLoaded() )
			Free();
		if ( IsFound() )
			Lose();
	}

	DWORD Size() const
	{
		ASSERT( IsFound() );
		return ::SizeofResource( Inst(), hRsrc );
    }

	void Find( LPCSTR lpszName, LPCSTR lpszType )
	{
		hRsrc = ::FindResource( Inst(), lpszName, lpszType );
		if ( !IsFound() )
			THROW( xAPIFail, "FindResource" );
	}

	void Lose()
	{
		ASSERT( !IsLoaded() && !IsLocked() );
		hRsrc = 0;
	}

	void Load()
	{
		ASSERT( IsFound() );
		hGlobal = ::LoadResource( Inst(), hRsrc );
		if ( !IsLoaded() )
			THROW( xAPIFail, "LoadResource" );
	}

	void Load( LPCSTR lpszName, LPCSTR lpszType )
	{
		ASSERT( !IsLoaded() && !IsFound() );
		Find( lpszName, lpszType );
		Load();
	}

	void Lock()
	{
		ASSERT( !IsLocked() && IsLoaded() );
		lpRes = ::LockResource( hGlobal );
		if ( !IsLocked() )
			THROW( xAPIFail, "LockResource" );
	}

	void Free()
	{
		ASSERT( IsLoaded() && !IsLocked() );
		if ( ::FreeResource( hGlobal ) )
			hGlobal = 0;
		else
			THROW( xAPIFail, "FreeResource" );
	}

	void Unlock()
	{
		ASSERT( IsLocked() );

#if !defined( __WIN32__ ) && !defined( _WIN32 )
		BOOL f = !::UnlockResource( hGlobal );
#else
		BOOL f = TRUE;
#endif			
		if ( f )
			lpRes = 0;
		else
			THROW( xAPIFail, "UnlockResource" );
			
	}

	#ifdef _DEBUG
	void AssertValid() const;
	#endif

};

#endif
