//---------------------------------------------------------------------------
//
//	File:			SURF.CPP
//	Programmer:		Bob Provencher
//	Create Date:	6/26/95
//	Last Mod Date:	4/26/96
//	Description:	DirectDraw surface initialization, tiling
//
//	Copyright (c) 1995, 1996 Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "SysInc.h"
#pragma hdrstop

#ifndef __COMMON_H__
#include "Common.h"
#endif

#ifndef __SURF_H__
#include "Surf.h"
#endif

#ifndef __DDDIAG_H__
#include "DDDiag.h"
#endif

#ifndef __TILETEST_H__
#include "TileTest.h"
#endif

#ifndef __LOGIDPAL_H__
#include "LogIdPal.h"
#endif

#ifndef __TILETEST_RH__
#include "TileTest.rh"
#endif

#ifndef __DIB_H__
#include "Dib.h"
#endif

#ifndef __DDOBJ_H__
#include "DDObj.h"
#endif

#ifndef __DDSURF_H__
#include "DDSurf.h"
#endif

#ifndef __DDPAL_H__
#include "DDPal.h"
#endif

#ifndef __DDCLIP_H__
#include "DDClip.h"
#endif

//#ifdef __BORLANDC__
//#pragma inline
//#endif

DDPalette				Palette;			// the palette

DDClipper				Clipper;			// DirectDraw clipper for window

DDSURFACEDESC			ddFlipSurfDesc;		// complex flipping structure
DDSurface				FlipSurface;		// flip surface root

DDSURFACEDESC			ddPrimSurfDesc;		// primary surface description
DDSurface				PrimarySurface;		// primary surface (the screen)

DDSURFACEDESC			ddOffSurfDesc;		// off-screen surface desc
DDSurface				OffSurface;			// off-screen surface

DDSURFACEDESC			ddOffSurf2Desc;		// off-screen surface desc #2
DDSurface				OffSurface2;		// off-screen surface #2

DDSURFACEDESC			ddTileSurfDesc;		// tile surface desc
DDSurface				TileSurface;		// tile surface

DDSURFACEDESC			ddTextSurfDesc;		// text surface desc
DDSurface				TextSurface;		// text surface

DDCOLORKEY				ddck;				// color key
DDPIXELFORMAT 			ddpf;				// pixel format for non 8 bpp modes

UINT					uPendingMoves;		//
HDC						hDCWork;			// use for setting bitmaps

DWORD start, end, attempts, frame, misses;

int  desired_fps;
BOOL page_flip;
int  numBackBuffs;

Map map( 1, 1 );

void RestoreIfLost( DDSurface& DirectDrawSurface )
{
	if ( DirectDrawSurface )
	{
		hResult = DirectDrawSurface->IsLost();
		if ( hResult == DDERR_SURFACELOST )
			hResult = DirectDrawSurface->Restore();
		ASSERT_SUCCESS( hResult );
	}
}

void CreatePrimarySurface( void )
{

	DWORD start = timeGetTime();

	if ( page_flip )
	{

		ddFlipSurfDesc.dwSize			= sizeof( DDSURFACEDESC );
		ddFlipSurfDesc.dwFlags 		  	= DDSD_CAPS |
										  DDSD_BACKBUFFERCOUNT;
		ddFlipSurfDesc.ddsCaps.dwCaps 	= DDSCAPS_COMPLEX |
										  DDSCAPS_FLIP |
										  DDSCAPS_PRIMARYSURFACE;

		//
		//	use n back buffers
		//

		ddFlipSurfDesc.dwBackBufferCount = numBackBuffs;

		DirectDraw.CreateSurface( ddFlipSurfDesc, FlipSurface );

		//
		//	get the pointers to the back buffers
		//

		hResult = FlipSurface->EnumAttachedSurfaces( NULL,
											EnumAttachedSurfacesCallback );

		ASSERT_SUCCESS( hResult );

		PrimarySurface = FlipSurface;
		ddPrimSurfDesc = ddFlipSurfDesc;

	}
	else
	{

		//
		//	create an object for the display surface
		//

		ddPrimSurfDesc.dwSize  			= sizeof( DDSURFACEDESC );
		ddPrimSurfDesc.dwFlags 		  	= DDSD_CAPS;
		ddPrimSurfDesc.ddsCaps.dwCaps 	= DDSCAPS_PRIMARYSURFACE;

		DirectDraw.CreateSurface( ddPrimSurfDesc, PrimarySurface );

	    //
	    // if we're windowed create a clipper
		//

		if ( bWindowed )
		{

			DirectDraw.CreateClipper( Clipper );

			hResult = Clipper->SetHWnd( 0, hWndMain );

			ASSERT_SUCCESS( hResult );

			hResult = PrimarySurface->SetClipper( Clipper );

			ASSERT_SUCCESS( hResult );

		}

    }

	DWORD elapsed = timeGetTime() - start;

	TRACEDD( " " << elapsed << " ms in CreatePrimarySurface()\n" );

}

void CreateBackSurface( void )
{

	DWORD start = timeGetTime();

	if ( !page_flip )
    {

		//
		//	create an off-screen surface
		//

		ddOffSurfDesc.dwSize			= sizeof( DDSURFACEDESC );
		ddOffSurfDesc.dwFlags 		  	= DDSD_CAPS | DDSD_HEIGHT |
										  DDSD_WIDTH;
		ddOffSurfDesc.ddsCaps.dwCaps 	= DDSCAPS_OFFSCREENPLAIN;
		ddOffSurfDesc.dwWidth			= nSurfWidth;
		ddOffSurfDesc.dwHeight			= nSurfHeight;

		DirectDraw.CreateSurface( ddOffSurfDesc, OffSurface );

		ASSERT_SUCCESS( hResult );

		if ( numBackBuffs > 1 )
		{

			ddOffSurf2Desc.dwSize			= sizeof( DDSURFACEDESC );
			ddOffSurf2Desc.dwFlags 		  	= DDSD_CAPS | DDSD_HEIGHT |
											  DDSD_WIDTH;
			ddOffSurf2Desc.ddsCaps.dwCaps 	= DDSCAPS_OFFSCREENPLAIN;
			ddOffSurf2Desc.dwWidth			= nSurfWidth;
			ddOffSurf2Desc.dwHeight			= nSurfHeight;

			DirectDraw.CreateSurface( ddOffSurf2Desc, OffSurface2 );

			ASSERT_SUCCESS( hResult );

		}

	}

	DWORD elapsed = timeGetTime() - start;

	TRACEDD( " " << elapsed << " ms in CreateOffSurface()\n" );

}

void RestorePrimarySurface( void )
{
	RestoreIfLost( PrimarySurface );
	if ( Palette )
		PrimarySurface->SetPalette( Palette );
	if ( !page_flip )
	{
		RestoreIfLost( OffSurface );
		RestoreIfLost( OffSurface2 );
	}
}

HRESULT CALLBACK EnumAttachedSurfacesCallback( LPDIRECTDRAWSURFACE lpDDSurface,
											   LPDDSURFACEDESC lpDDSurfDesc,
											   LPVOID )
{

	//
	//	we only need the first back buffer
	//

	if ( lpDDSurfDesc->ddsCaps.dwCaps & DDSCAPS_BACKBUFFER )
	{
		OffSurface			= lpDDSurface;
		ddOffSurfDesc		= *lpDDSurfDesc;
		return DDENUMRET_CANCEL;
	}

	return DDENUMRET_OK;

}

static LogPalette pal;

void CreateTileSurface( void )
{

	//
	//	load the tile bitmap
	//

	Dib	tiles;

	if ( !TileSurface )
	{

		//
		//	create the tile surface
		//

		tiles.Load( string( "tiles.bmp" ) );

		ddTileSurfDesc.dwSize				= sizeof( DDSURFACEDESC );
		ddTileSurfDesc.dwFlags	 		  	= DDSD_CAPS | DDSD_CKSRCBLT |
											  DDSD_HEIGHT | DDSD_WIDTH;

		ddTileSurfDesc.ddsCaps.dwCaps 		= DDSCAPS_OFFSCREENPLAIN;

		if ( dwVidMem < ( 1024L * 1024L ) )
        {
        	TRACEDD( "Allocating tiles in system memory" );
			ddTileSurfDesc.ddsCaps.dwCaps |= DDSCAPS_SYSTEMMEMORY;
        }

		ddTileSurfDesc.dwWidth				= tiles.GetWidth();
		ddTileSurfDesc.dwHeight				= tiles.GetHeight();

		DirectDraw.CreateSurface( ddTileSurfDesc, TileSurface );

        TRACEDD( "TileSurface created.\n" );

	}
	else
	{
		RestoreIfLost( TileSurface );
	}

	if ( bpp == 8 )
	{

		if ( !Palette )
		{

			if ( !tiles.GetBitmap() )
				tiles.Load( string( "tiles.bmp" ) );

			pal = LogIdPalette( tiles );

			//
			//	create the main palette from the tiles palette
			//

			DirectDraw.CreatePalette( DDPCAPS_8BIT | DDPCAPS_INITIALIZE,
									   pal.Entries(), Palette );

			if ( Palette )
			{

				//
				//	set the primary surface's palette
				//

				if ( PrimarySurface )
				{
					hResult = PrimarySurface->SetPalette( Palette );
					ASSERT_SUCCESS( hResult );
				}

			}

		}

	}

	//
	//	copy the tile bitmap onto the tile surface
	//

	if ( !tiles.GetBitmap() )
		tiles.Load( string( "tiles.bmp" ) );

	if ( TileSurface )
	{

		hResult = TileSurface->GetDC( &hDCWork );

		ASSERT_SUCCESS( hResult );

		StretchDIBits( hDCWork, 0, 0, tiles.GetWidth(), tiles.GetHeight(),
				   	   0, 0, tiles.GetWidth(), tiles.GetHeight(),
				   	   tiles.GetBitmap(), tiles.GetBitmapInfo(),
				   	   DIB_RGB_COLORS, SRCCOPY );

		GdiFlush();

		hResult = TileSurface->ReleaseDC( hDCWork );

		ASSERT_SUCCESS( hResult );

		GdiFlush();

		hDCWork = 0;

	}

}

void CreateTextSurface( void )
{

	if ( !TextSurface )
	{

		//
		//	set up the text surface
		//

		ddTextSurfDesc.dwSize				= sizeof( DDSURFACEDESC );
		ddTextSurfDesc.dwFlags				= DDSD_CAPS | DDSD_CKSRCBLT |
											  DDSD_HEIGHT | DDSD_WIDTH;

		ddTextSurfDesc.ddsCaps.dwCaps 		= DDSCAPS_OFFSCREENPLAIN;

		if ( dwVidMem < 1024L * 1024L )
			ddTextSurfDesc.ddsCaps.dwCaps |= DDSCAPS_SYSTEMMEMORY;

		ddTextSurfDesc.dwWidth				= TEXT_WIDTH;
		ddTextSurfDesc.dwHeight				= TEXT_HEIGHT;

		ddTextSurfDesc.ddckCKSrcBlt.dwColorSpaceLowValue  =
		ddTextSurfDesc.ddckCKSrcBlt.dwColorSpaceHighValue = 0;

		DirectDraw.CreateSurface( ddTextSurfDesc, TextSurface );

	}
	else
	{
		RestoreIfLost( TextSurface );
	}

	//
	//	set the transparent color (black) when text surface used
	//	as source in a blt.
	//	(CreateSurface apparently isn't recognizing DDSD_CKSRCBLT)
	//

	hResult = TextSurface->SetColorKey( DDCKEY_SRCBLT,
									   &ddTextSurfDesc.ddckCKSrcBlt );

	ASSERT_SUCCESS( hResult );

}

void CreateSurfaces( void )
{
	CreatePrimarySurface();
	CreateTileSurface();
    CalcLayout( nClientWidth, nClientHeight );
    CreateBackSurface();
	CreateTextSurface();
}

void CleanUp( void )
{

	bRunning = FALSE;

	if ( page_flip )
	{
		if ( DirectDraw )
			DirectDraw->FlipToGDISurface();
	}

}

void ShowFPS( void )
{

	char buffer[ 40 ];

	if ( frame <= 1 )
		return;

	wsprintf( buffer, "%d BPP, %ld FPS", (int)bpp, ( frame * 1000 ) / ( end - start ) );

    TRACEDD( buffer << "\n" );

	if ( TextSurface->IsLost() )
		CreateTextSurface();

	//
	//	copy the text bitmap onto the text surface
	//

	if ( TextSurface )
	{

		hResult = TextSurface->GetDC( &hDCWork );

		ASSERT_SUCCESS( hResult );

		PatBlt( hDCWork, 0, 0, TEXT_WIDTH, TEXT_HEIGHT, BLACKNESS );

        COLORREF color = SetTextColor( hDCWork, RGB( 0, 255, 0 ) );
        int mode = SetBkMode( hDCWork, TRANSPARENT );
		TextOut( hDCWork, 0, 0, buffer, strlen( buffer ) );
        SetTextColor( hDCWork, color );
        SetBkMode( hDCWork, mode );

		GdiFlush();

		hResult = TextSurface->ReleaseDC( hDCWork );

		ASSERT_SUCCESS( hResult );

		GdiFlush();

		hDCWork = 0;

		hResult = OffSurface->BltFast( 0, nClientHeight - TEXT_HEIGHT,
									  TextSurface, NULL,
									  DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY );

        //
        //	bug in ATI Mach64 driver for DirectX I ??
        //	SRC color key doesn't work after a surface is released.
        //

    	if ( hResult == DDERR_INVALIDPARAMS )
        {
			hResult = OffSurface->BltFast( 0, nClientHeight - TEXT_HEIGHT,
									  TextSurface, NULL,
									  DDBLTFAST_WAIT );
        }

		ASSERT_SUCCESS( hResult );

	}

}

void ComposeImage( void )
{

	DWORD start = timeGetTime();

	if ( OffSurface->IsLost() )
		RestorePrimarySurface();

    Tile();
	ShowFPS();

	TRACEDD( ( timeGetTime() - start ) << " ms in ComposeImage()\n" );

}

static int tile_width;
static int tile_height;
static int width;
static int height;
static int tiles_x;
static int tiles_y;

void CalcLayout( int cx, int cy )
{

	//
    //	tile width & height must be a power of 2
    //

	tile_width   = ddTileSurfDesc.dwWidth;
	tile_height  = ddTileSurfDesc.dwHeight / 3;

    TRACEDD( "Tile width: " << tile_width  << ", Tile height: "
    			<< tile_height << "\n" );

    if ( tile_width == 0 )
    	return;

	width  = cx;
    nSurfWidth  = ( cx + tile_width  - 1 ) & ~( tile_width  - 1 );
    
	height = cy;
    nSurfHeight = ( cy + tile_height - 1 ) & ~( tile_height - 1 );

    TRACEDD( "Window width: " << width  << ", Window height: "
    			<< height << "\n" );

    TRACEDD( "Surface width: " << nSurfWidth << ", Surface height: "
    			<< nSurfHeight << "\n" );

	tiles_x = nSurfWidth  / tile_width;
	tiles_y = nSurfHeight / tile_height;

    TRACEDD( "Tiles X: " << tiles_x  << ", Tiles Y: "
    			<< tiles_y << "\n" );

}

void Tile( void )
{

	DWORD start, end;

    start = timeGetTime();

	if ( TileSurface->IsLost() )
		CreateTileSurface();

    end = timeGetTime();

    TRACEDD( ( end - start ) << " ms in CreateFloorSurface()\r\n" );

	int man_x, man_y;
	int start_x, start_y;

    map.GetMan( man_x, man_y );

    start_x = man_x - ( tiles_x / 2 );
    start_y = man_y - ( tiles_y / 2 );

    constrain( start_x, 0, map.Width()  - tiles_x );
    constrain( start_y, 0, map.Height() - tiles_y );

    hResult = TileSurface->Lock( NULL, &ddTileSurfDesc,
    							  DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,
                                  0 );

    ASSERT_SUCCESS( hResult );

    hResult = OffSurface->Lock( NULL, &ddOffSurfDesc,
    							  DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,
                                  0 );

    ASSERT_SUCCESS( hResult );

    BYTE* tiles = (BYTE*)ddTileSurfDesc.lpSurface;
    BYTE* surf  = (BYTE*)ddOffSurfDesc.lpSurface;

    BYTE* source;
    BYTE* dest;

    int source_width = ddTileSurfDesc.dwWidth * bytes_per_pixel;

    TRACEDD( "Source width: " << source_width << '\n' );

    int dest_pitch   = ddOffSurfDesc.lPitch;

    TRACEDD( "Dest pitch: " << dest_pitch << '\n' );

    int dest_offset  = dest_pitch - source_width;

    int tile;

    int i, j, line, x;

    //
    //	optimize this nested loop
    //

    start = timeGetTime();

    for ( i = 0; i < tiles_x; i++ )
    {
        dest   = surf + ( i * source_width );
    	for ( j = 0; j < tiles_y; j++ )
        {
        	tile = map.GetTile( i + start_x, j + start_y );
	        source = tiles + ( tile * source_width * tile_height );
            for ( line = 0; line < tile_height; line++ )
            {
				for ( x = 0; x < source_width; x += sizeof( int ) )
                {
                	*(int*)dest = *(int*)source;
					dest   += sizeof( int );
					source += sizeof( int );
                }
                dest += dest_offset;
            }
        }
    }

    end = timeGetTime();

    TRACEDD( ( end - start ) << " ms tiling\n" );

    hResult = OffSurface->Unlock( ddOffSurfDesc.lpSurface );

    ASSERT_SUCCESS( hResult );

    hResult = TileSurface->Unlock( ddTileSurfDesc.lpSurface );

    ASSERT_SUCCESS( hResult );

}

void ShowImage( void )
{

	DWORD s = timeGetTime();

	if ( page_flip )
	{

		hResult = FlipSurface->Flip( NULL, DDFLIP_WAIT );

		ASSERT_SUCCESS( hResult );

	}
	else
	{

		hResult = PrimarySurface->Blt( &rcWnd,
									  OffSurface,
									  &rcWndClient,
									  DDBLT_WAIT,
									  NULL );

		ASSERT_SUCCESS( hResult );

		if ( numBackBuffs > 1 )
			swap( OffSurface, OffSurface2 );

	}

	TRACEDD( ( timeGetTime() - s ) << " ms in ShowImage()\n" );

	frame++;

}

void ComposeAndShowImage( void )
{

	if ( !bRunning )
    	return;

	if ( start == 0 )
		start = timeGetTime();

	ComposeImage();
	ShowImage();

	end = timeGetTime();

	TRACEDD( frame << " frames.\n" );
	TRACEDD( ( frame * 1000 / ( end - start ) ) << " fps.\n" );

}

