//---------------------------------------------------------------------------
//
//	File:			TILETEST.CPP
//	Programmer:		Bob Provencher
//	Create Date:	6/26/95
//	Last Mod Date:	4/26/96
//	Description:	Direct Draw sample program
//
//	Copyright (c) 1995, 1996 Aesir Software, Inc.
//	Copyright (c) 1996 Cog Interactive
//	All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "SysInc.h"
#pragma hdrstop

#ifndef __COMMON_H__
#include "Common.h"
#endif

#ifndef __TILETEST_H__
#include "TileTest.h"
#endif

#ifndef __SURF_H__
#include "Surf.h"
#endif

#ifndef __DDIAG_H__
#include "DDDiag.h"
#endif

#ifndef __TILETEST_H__
#include "TileTest.rh"
#endif

#ifndef __DDOBJ_H__
#include "DDObj.h"
#endif

#ifndef __MAP_H__
#include "Map.h"
#endif

HINSTANCE	hInst;
HINSTANCE	hPrevInst;
LPCSTR      lpszCmdLine;
int			nCmdShow;

LPCSTR		lpszMainWndClass	= "MainWndClass";
LPCSTR		lpszDirectDraw		= "TileTest Direct Draw Sample";

RECT		rcWnd, rcWndClient;
HWND		hWndMain;
BOOL		fActive;

int			nScreenWidth, nScreenHeight;
int			nClientWidth, nClientHeight;
int			nSurfWidth,   nSurfHeight;

BOOL		bTrace;
BOOL		bWindowed;
BOOL		bDirectDrawInit = FALSE;
DWORD		dwFreq;
DWORD		dwVidMem;

BOOL		bRunning;

int			bpp;
int			bytes_per_pixel;
int			period, last, now;

DDObject 	DirectDraw;		// Direct Draw object

DDCAPS		ddCaps;

const char* lpszIniSecName = "TileTest";

int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
					LPSTR lpszCmd, int nCmd )
{

	hInst 		= hInstance;
    hPrevInst	= hPrevInstance;
    lpszCmdLine	= lpszCmd;
    nCmdShow	= nCmd;

	int rc;

    try
    {
    	rc = Run();
    }
    catch ( Exception* x )
    {
		if ( page_flip && FlipSurface )
			DirectDraw->FlipToGDISurface();
		x->MessageBox();
		rc = -1;
    }

    return rc;

}

int Run()
{

	MSG msg;

	randomize();

	if ( !hPrevInst )
		if ( !InitApplication( hInst ) )
		{
			TRACE( "InitApplication() failed.\n" );
			return 0;
		}

	if ( !InitInstance( hInst, nCmdShow ) )
	{
		TRACE( "InitInstance() failed.\n" );
		CleanUp();
		return 0;
	}

	bRunning = TRUE;

    last = timeGetTime();

	do
	{
		if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
		{
			if ( msg.message != WM_QUIT )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
		}
		else
		{
			if ( AppIdle() )
				WaitMessage();
		}
	}
	while ( msg.message != WM_QUIT );

	return msg.wParam;

}

void WritePrivateProfileInt( LPCSTR section, LPSTR key, int value, LPSTR file )
{
	char buffer[ 20 ];
	wsprintf( buffer, "%d", value );
    WritePrivateProfileString( section, key, buffer, file );
}

BOOL LoadIniSettings( void )
{

	char buffer[ 512 ];

	if ( GetModuleFileName( hInst, buffer, 512 ) == 0 )
		return FALSE;

	strcpy( buffer + strlen( buffer ) - 3, "INI" );

	nScreenWidth  = GetPrivateProfileInt( lpszIniSecName, "ScreenWidth",  640, buffer );
	nScreenHeight = GetPrivateProfileInt( lpszIniSecName, "ScreenHeight", 480, buffer );
	numBackBuffs  = GetPrivateProfileInt( lpszIniSecName, "BackBuffers", 2, buffer );
	desired_fps   = GetPrivateProfileInt( lpszIniSecName, "DesiredFrameRate", 60, buffer );
	page_flip     = GetPrivateProfileInt( lpszIniSecName, "PageFlip", 0, buffer );
	bTrace        = GetPrivateProfileInt( lpszIniSecName, "Trace", 1, buffer );
	bWindowed     = GetPrivateProfileInt( lpszIniSecName, "Windowed", 1, buffer );

	//
	//  constrain these variables to min and max
	//

	constrain( nScreenWidth,  320, 800 );
	constrain( nScreenHeight, 200, 600 );
	constrain( numBackBuffs,  1,   2 );
	constrain( desired_fps,   1,   87 );

	period = ( 1000 / desired_fps ) - 1;

	SETTRACE( bTrace );

	WritePrivateProfileInt( lpszIniSecName, "ScreenWidth",  nScreenWidth,  buffer );
	WritePrivateProfileInt( lpszIniSecName, "ScreenHeight", nScreenHeight, buffer );
	WritePrivateProfileInt( lpszIniSecName, "BackBuffers",  numBackBuffs,  buffer );
	WritePrivateProfileInt( lpszIniSecName, "DesiredFrameRate", desired_fps, buffer );
	WritePrivateProfileInt( lpszIniSecName, "PageFlip", page_flip, buffer );
	WritePrivateProfileInt( lpszIniSecName, "Trace", bTrace, buffer );
	WritePrivateProfileInt( lpszIniSecName, "Windowed", bWindowed, buffer );

	return TRUE;

}

//
//	AppIdle returns a BOOLean that specifies whether or not it's ok to yield
//

BOOL AppIdle( void )
{
   	now = timeGetTime();
    if ( now - last > period )
    {
     	last = now;
       	ComposeAndShowImage();
	}
	return FALSE;
}

BOOL InitApplication( HINSTANCE hInstance )
{

	WNDCLASS  wc;

	wc.style			= CS_BYTEALIGNCLIENT | CS_DBLCLKS;
	wc.lpfnWndProc		= MainWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon   			= LoadIcon( NULL, IDI_APPLICATION );
	wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground	= NULL;
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= lpszMainWndClass;

	return RegisterClass( &wc );

}

void GenerateMap()
{
	map.Generate( MAP_WIDTH, MAP_HEIGHT );
}

BOOL InitInstance( HINSTANCE hInstance, int nCmdShow )
{

	hInst = hInstance;

	if ( !LoadIniSettings() )
		return FALSE;

	if ( bWindowed && page_flip )
	{
		TRACEDD( "Warning: can't flip in non-exclusive mode.\n" );
     	page_flip = FALSE;
	}

//	if ( bWindowed && numBackBuffs > 1 )
//	{
//		TRACEDD( "Warning: can't have more than 1 back buffer in non-exclusive mode.\n" );
//     	numBackBuffs = 1;
//	}

	if ( bWindowed )
	{

		HDC hDC = GetDC( NULL );
		bpp = (BYTE)( GetDeviceCaps( hDC, PLANES ) * GetDeviceCaps( hDC, BITSPIXEL ) );
		ReleaseDC( NULL, hDC );

		if ( bpp < 8 )
		{
			TRACEDD( "Warning: windowed mode requires at least a 256 color video driver.\n" );
			bWindowed = FALSE;
 		}

        bytes_per_pixel = ( ( bpp + 1 ) & ~1 ) / 8;

	}
	else
	{
		bpp = 8;
        bytes_per_pixel = 1;
	}

	TRACEDD( "Display is " << (int)bpp << " bpp.\n" );

    GenerateMap();

	DWORD dwStyle = WS_EX_TOPMOST | WS_POPUP;

	int x, y;

	x = 0;
	y = 0;

	rcWnd.left   = x;
	rcWnd.top    = y;
	rcWnd.right  = x + nScreenWidth;
	rcWnd.bottom = y + nScreenHeight;

	rcWndClient = rcWnd;

    //
    //	if tiled use the default window position
    //

	if ( bWindowed )
	{
		dwStyle = WS_OVERLAPPEDWINDOW;
		x = nClientWidth = CW_USEDEFAULT;
		nScreenWidth  = GetSystemMetrics( SM_CXSCREEN );
		nScreenHeight = GetSystemMetrics( SM_CYSCREEN );
	}

	hWndMain = CreateWindow( lpszMainWndClass, lpszDirectDraw,
							 dwStyle,
							 x, y,
							 nClientWidth, nClientHeight,
							 NULL, NULL, hInstance, NULL );

	if ( !hWndMain )
		return FALSE;

	ShowWindow( hWndMain, nCmdShow );
	UpdateWindow( hWndMain );

	SetFocus( hWndMain );

	GetClientRect( hWndMain, &rcWndClient );

	nClientWidth  = rcWndClient.right  - rcWndClient.left;
	nClientHeight = rcWndClient.bottom - rcWndClient.top;

    try
    {
        CalcLayout( nClientWidth, nClientHeight );
		InitDirectDraw();
        bDirectDrawInit = TRUE;
    }
    catch ( Exception* x )
    {
		DestroyWindow( hWndMain );
		x->MessageBox();
        return FALSE;
    }

    return TRUE;

}

HRESULT CALLBACK EnumDisplayModesCallback( LPDDSURFACEDESC lpDDSurfDesc,
										   LPVOID )
{
	TRACEDD( "Display Mode: "
		  	 << "\n Width:             " << lpDDSurfDesc->dwWidth
		  	 << "\n Height:            " << lpDDSurfDesc->dwHeight
		  	 << "\n Bit Depth:         "
		  	 << lpDDSurfDesc->ddpfPixelFormat.dwRGBBitCount
		  	 << endl << endl );
	return DDENUMRET_OK;
}

void InitDirectDraw( void )
{

	//
	//	create our main DirectDraw object
	//

	DirectDraw.Create();

	DWORD dwFlags = DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_ALLOWMODEX;

	if ( bWindowed )
    	dwFlags = DDSCL_NORMAL;

	hResult = DirectDraw->SetCooperativeLevel( hWndMain, dwFlags );

	ASSERT_SUCCESS( hResult );

	DirectDraw->EnumDisplayModes( 0, 0, 0, EnumDisplayModesCallback );

	ASSERT_SUCCESS( hResult );

	if ( !bWindowed )
	{
		hResult = DirectDraw->SetDisplayMode( nScreenWidth, nScreenHeight, 8 );
		ASSERT_SUCCESS( hResult );
		bpp = 8;
	}

    //
    //	check capabilites
    //

    ddCaps.dwSize = sizeof( DDCAPS );

    hResult = DirectDraw->GetCaps( &ddCaps, NULL );

	ASSERT_SUCCESS( hResult );

	dwVidMem = ddCaps.dwVidMemTotal;

	//
	//	This didn't seem to be supported in DirectDraw 4.02
	//

	hResult = DirectDraw->GetMonitorFrequency( &dwFreq );

	if ( SUCCEEDED( hResult ) )
		TRACEDD( "Monitor frequency is " << dwFreq / 1000 << "Hz.\n" );

	CreateSurfaces();

}

LPARAM CALLBACK MainWndProc( HWND hWnd, UINT message, WPARAM wParam,
							 LPARAM lParam )
{

	switch ( message )
	{

		case WM_CREATE:

			hWndMain = hWnd;

    		if ( !bWindowed )
				ShowCursor( FALSE );

			return 0;

		case WM_ACTIVATEAPP:

			fActive = wParam;

			return 0;

        case WM_KEYDOWN:

        	if ( (int)wParam == VK_ESCAPE )
            	DestroyWindow( hWndMain );

        	return 0;

		case WM_LBUTTONDBLCLK:

			DestroyWindow( hWndMain );

			return 0;

		case WM_ERASEBKGND:

			return TRUE;

		case WM_PAINT:

			PAINTSTRUCT ps;
			BeginPaint( hWnd, &ps );
			EndPaint( hWnd, &ps );
			return TRUE;

		case WM_SIZE:

			nClientWidth  = LOWORD( lParam );
			nClientHeight = HIWORD( lParam );

            CalcLayout( nClientWidth, nClientHeight );

            if ( bDirectDrawInit )
            {
				CreateBackSurface();
            }

            // fall through

		case WM_MOVE:

			if ( bWindowed )
			{

				GetClientRect( hWnd, &rcWndClient );
				rcWnd = rcWndClient;
				ClientToScreen( hWnd, &rcWnd );

			}

			break;

        case WM_MOUSEMOVE:

        	break;

		case WM_DESTROY:

			CleanUp();

			if ( !bWindowed )
				ShowCursor( TRUE );

			PostQuitMessage( 0 );

			return 0;

        case WM_NCDESTROY:

			hWndMain = 0;

            return 0;

	}

	return DefWindowProc( hWnd, message, wParam, lParam );

}

